'use strict';

class Analytics {
  constructor() {
    this.lastTrackedPage_ = null;

    this.initialize();
  }

  countPage(ids) {
    if (Array.isArray(ids)) {
      ids = ids.join('/');
    }

    ids = '/' + ids;

    if (this.lastTrackedPage_ === ids) {
      return;
    }

    this.lastTrackedPage_ = ids;

    ga('send', 'pageview', {
      'page': ids
    });
  }


  countEvent(category, action, label) {
    var data = {};

    if (category) {
      data.eventCategory = category;
    }

    if (action) {
      data.eventAction = action;
    }

    if (label) {
      data.eventLabel = label;
    }

    ga('send', 'event', data);
  }

  initialize() {
    window['GoogleAnalyticsObject'] = 'ga';
    window.ga = window.ga || function() {
          (window.ga.q = window.ga.q || []).push(arguments);
        };
    window.ga.l = Date.now();

    var script = document.createElement('script');
    script.async = 1;
    script.src = 'https://www.google-analytics.com/analytics.js';
    var firstScript = document.getElementsByTagName('script')[0];
    firstScript.parentNode.insertBefore(script, firstScript);

    var code = App.browserType === App.BROWSER_TYPE_OPERA ?
        Analytics.CODE_OPERA : Analytics.CODE_CHROME;

    ga('create', code, 'auto');
    ga('set', 'checkProtocolTask', null);
  }
};

Analytics.CODE_CHROME = 'UA-22616710-10';
Analytics.CODE_OPERA = 'UA-22616710-11';

Analytics.EVENT_DOUBLE_PROXY = 'DoubleProxy';
Analytics.EVENT_NOT_HANDLED = 'NotHandled';
Analytics.EVENT_NOT_TRANSLATED = 'NotTranslated';
Analytics.EVENT_LOCAL_ERROR_CODE = 'ErrorCode';
Analytics.EVENT_COMMUNICATION_ERROR = 'CommunicationError';