'use strict';

class BrowsingActivity extends Base {
  constructor() {
    super();
  }

  registerListeners() {
    super.registerListeners();

    chrome.webRequest.onCompleted.addListener(
        this.onCompleted_.bind(this),
        {'urls': [BrowsingActivity.PATTERN_REMOVE_DEVICE]});
    chrome.webRequest.onBeforeRequest.addListener(
        this.onBeforeRequest_.bind(this),
        {'urls': [
          BrowsingActivity.PATTERN_DATA_PLAN_OVER_BANDWIDTH,
          BrowsingActivity.PATTERN_DATA_PLAN_EXPIRED,
          BrowsingActivity.PATTERN_DATA_PLAN_PURCHASED
        ]}, ['blocking']);
    chrome.webRequest.onHeadersReceived.addListener(
        this.onBeforeRedirect_.bind(this),
        {'urls': ['<all_urls>']}, ["blocking", "responseHeaders"]);
  }

  initialize() {
    super.initialize();
  }

  onCompleted_(details) {
    if (this.isMatching_(details.url, BrowsingActivity.PATTERN_REMOVE_DEVICE)) {
      this.onDeviceRemovedAction_();
    }
  }

  onBeforeRequest_(details) {
    var response = {};
    var isOverBandwidthPage = this.isMatching_(details.url,
        BrowsingActivity.PATTERN_DATA_PLAN_OVER_BANDWIDTH);
    var isExpiredPage = this.isMatching_(details.url,
        BrowsingActivity.PATTERN_DATA_PLAN_EXPIRED);
    var isPurchasedPage = this.isMatching_(details.url,
        BrowsingActivity.PATTERN_DATA_PLAN_PURCHASED);

    if (isOverBandwidthPage || isExpiredPage) {
      if (details.type !== 'main_frame' ||
          Url.hasParam(details.url, Url.PARAM_DEVICE_ID)) {
        if (isOverBandwidthPage) {
          this.onDataPlanOverBandwidth_();
        } else {
          this.onDataPlanExpired_();
        }
      } else {
        Url.openMiniInternetTab(device.id, details.url);
      }
    } else if (isPurchasedPage) {
      this.onDataPlanPurchased_();
    }

    return response;
  }

  onBeforeRedirect_(details) {
    if (details && details.responseHeaders) {
      var isUnknownHost = details.responseHeaders.some(function(header) {
        return header.name === 'Location' &&
            header.value.startsWith(Url.UNKNOWN_HOST);
      });

      if (isUnknownHost) {
        var list = proxy.bypassList;
        try {
          var hostname = (new URL(details.url)).hostname;
          if (proxy.bypassList.indexOf(hostname) === -1) {
            proxy.bypassList = list.concat([hostname]);
          }

          return {
            redirectUrl: details.url
          }
        } catch (e) {

        }
      }
    }
  }

  onDataPlanOverBandwidth_() {
    BrowsingActivity.Events.onDataPlanOverBandwidth.dispatch();
    this.onAction_(BrowsingActivity.ACTION_DATA_PLAN_OVER_BANDWIDTH);
  }

  onDataPlanExpired_() {
    BrowsingActivity.Events.onDataPlanExpired.dispatch();
    this.onAction_(BrowsingActivity.ACTION_DATA_PLAN_EXPIRED);
  }

  onDataPlanPurchased_() {
    BrowsingActivity.Events.onDataPlanPurchased.dispatch();
    this.onAction_(BrowsingActivity.ACTION_DATA_PLAN_PURCHASED);
  }

  onDeviceRemovedAction_() {
    BrowsingActivity.Events.onDeviceRemovedAction.dispatch();
    this.onAction_(BrowsingActivity.ACTION_DEVICE_REMOVED);
  }

  onAction_(id) {
    BrowsingActivity.Events.onAction.dispatch(id);
  }

  isMatching_(url, pattern) {
    pattern = new RegExp(pattern.replace(/[*]/g, '.*'), 'gi');
    return pattern.test(url);
  }
};

BrowsingActivity.ACTION_DEVICE_REMOVED = 0;
BrowsingActivity.ACTION_DATA_PLAN_OVER_BANDWIDTH = 1;
BrowsingActivity.ACTION_DATA_PLAN_EXPIRED = 2;
BrowsingActivity.ACTION_DATA_PLAN_PURCHASED = 3;

BrowsingActivity.PATTERN_REMOVE_DEVICE =
    Url.USER_DASHBOARD + '/*/remove_device/*';
BrowsingActivity.PATTERN_REMOVE_PLAN = Url.DOWNGRADE_PAGE + '/*';
BrowsingActivity.PATTERN_DATA_PLAN_OVER_BANDWIDTH =
    Url.PLAN_OVER_BANDWIDTH + '*';
BrowsingActivity.PATTERN_DATA_PLAN_EXPIRED = Url.PLAN_EXPIRED + '*';
BrowsingActivity.PATTERN_DATA_PLAN_PURCHASED = Url.USER_DASHBOARD +
    '/*new_plan_success=true*';

BrowsingActivity.Events = {
  onAction: new chrome.Event(),
  onDeviceRemovedAction: new chrome.Event(),
  onDataPlanOverBandwidth: new chrome.Event(),
  onDataPlanExpired: new chrome.Event(),
  onDataPlanPurchased: new chrome.Event()
};

window.BrowsingActivity = BrowsingActivity;
