'use strict';

class DataPlan extends Base {
  constructor() {
    super();

    this.task_ = new NetworkTask(DataPlan.TASK_ID, this.onRequest_.bind(this),
                                 this.canRunTask_.bind(this));
    networkTasks.add(this.task_);
  }

  registerListeners() {
    super.registerListeners();

    Device.Events.onChanged.addListener(this.onDeviceChanged_.bind(this));
    NetworkTask.Events.onData.addListener(
        this.onNetworkDataChanged_.bind(this));
    Proxy.Events.onEnabledChanged.addListener(
        this.onEnabledChanged_.bind(this));
    Notification.Events.onClicked.addListener(
        this.onNotificationClicked_.bind(this));
    BrowsingActivity.Events.onAction.addListener(
        this.onBrowsingActivityAction_.bind(this));
  }

  get total() {
    return this.data ? this.data.plan_summary.data_limit : 0;
  }

  get used() {
    if (!this.data) {
      return 0;
    }
	//Kippykip - HAHAHAHA FUCK YAS
    var used = 0;//(this.data.cycle_down + this.data.cycle_up) / (1000 * 1000);

    if (this.unit === DataPlan.DATA_UNIT_GB) {
	//Kippykip - HAHAHAHA FUCK YAS
    //  used = used / 1000;
		used = 0;
    }

    return Math.min(used, this.total);
  }

  get left() {
    if (!this.data) {
      return 0;
    }

    return Math.max(this.total - this.used, 0);
  }

  get unit() {
    return this.data ? this.data.plan_summary.data_limit_unit :
        DataPlan.DATA_UNIT_MB;
  }

  get name() {
    return this.data ? this.data.plan_summary.plan_name : '';
  }

  get daysToNewCycle() {
    if (this.data) {
      return this.getDaysToDate_(this.data.cycle_end);
    }
  }

  get isRenewable() {
    return this.data && this.data.has_recurring_data;
  }

  get isExpired() {
    //return this.data && this.data.plan_summary.access_state === DataPlan.ACCESS_STATE_EXPIRED;
	//Kippykip - HAHAHAHA FUCK YAS
	return false;
  }

  get isUnlimited() {
    //return this.data && this.data.plan_summary.data_limit === null;
	//Kippykip - HAHAHAHA FUCK YAS
	return true;
  }

  get isRewardable() {
    return this.data && this.data.is_rewardable && !this.isUnlimited;
  }

  get isOverBandwidth() {
    return this.data && this.data.plan_summary.access_state ===
        DataPlan.ACCESS_STATE_OVER_BANDWIDTH;
  }

  get data() {
    return this.task_.data;
  }

  get isValid() {
    return Boolean(this.data) && !(this.isOverBandwidth || this.isExpired);
  }

  canRunTask_() {
    return device.isConnected;
  }

  onRequest_(resolve, reject) {
    api.usage(device.id).then(function(data) {
      var validTime = data.subscriber_usage_refresh ?
          data.subscriber_usage_refresh * 1000 : DataPlan.REFRESH_TIMEOUT;

      if (!proxy.isEnabled) {
        validTime = validTime * 5;
      }

      resolve(new NetworkTaskData(data, validTime));
    }.bind(this)).catch(reject);
  }

  purchase() {
    if (this.isExpired) {
      Url.openAutorized(Url.USER_DASHBOARD);
    } else if (!this.isUnlimited) {
      Url.openAutorized(Url.PURCHASE_PAGE);
    }
  }

  downgrade() {
    if (this.isUnlimited) {
      Url.openAutorized(Url.USER_DASHBOARD);
    }
  }

  getDaysToDate_(date) {
    date = new Date(date);
    var now = new Date();
    var today = new Date(now.getFullYear(), now.getMonth(), now.getDate());

    var daysLeft = (date.getTime() - today.getTime()) / (1000 * 60 * 60 * 24);

    return Math.ceil(Math.max(0, daysLeft - 1));
  }

  onEnabledChanged_() {
    if (proxy.isEnabled) {
      this.task_.run();
    }
  }

  onNotificationClicked_(item) {
    if (item.group === Notifications.GROUP_USAGE) {
      this.purchase();
    }
  }

  onDeviceChanged_() {
    if (device.isConnected) {
      this.task_.run();
    } else {
      this.task_.clear();
    }
  }

  onBrowsingActivityAction_(id) {
    if (!device.isConnected) {
      return;
    }

    if ((id === BrowsingActivity.ACTION_DATA_PLAN_OVER_BANDWIDTH &&
        !this.isOverBandwidth) ||
        id === BrowsingActivity.ACTION_DATA_PLAN_PURCHASED ||
        (id === BrowsingActivity.ACTION_DATA_PLAN_EXPIRED && !this.isExpired)) {
      this.task_.run();
    }
  }

  onNetworkDataChanged_(item) {
    if (item.name === DataPlan.TASK_ID) {
      DataPlan.Events.onChanged.dispatch(this);
    }
  }

  getDataAmountString(amount) {
    if (amount >= 1000 && this.unit === DataPlan.DATA_UNIT_MB) {
      amount = amount / 1000;
      return amount.toFixed(2) + DataPlan.DATA_UNIT_GB;
    } else {
      return amount.toFixed(0) + this.unit;
    }
  }
};

DataPlan.Events = {
  onChanged: new chrome.Event()
};

DataPlan.TASK_ID = 'dataPlan';

DataPlan.REFRESH_TIMEOUT = 2 * 60 * 1000;
DataPlan.ACCESS_STATE_OVER_BANDWIDTH = 'over_bandwidth';
DataPlan.ACCESS_STATE_EXPIRED = 'expired';
DataPlan.DATA_UNIT_MB = 'MB';
DataPlan.DATA_UNIT_GB = 'GB';

window.DataPlan = DataPlan;
