'use strict';

class Errors extends Base {
  constructor() {
    super();
    this.hasUnexpectedError_ = false;
    this.activeErrorsList_ = [];
  }

  registerListeners() {
    super.registerListeners();

    var onChanged_ = this.onChanged_.bind(this);
    Api.Events.onNetworkStateChanged.addListener(onChanged_);
    NetworkTasks.Events.onError.addListener(onChanged_);
    App.Events.onCookiesError.addListener(onChanged_);
    SessionCookie.Events.onChanged.addListener(onChanged_);
    OffRoad.Events.onChanged.addListener(onChanged_);
    User.Events.onChanged.addListener(onChanged_);
    Device.Events.onChanged.addListener(onChanged_);
    Proxy.Events.onControllableChanged.addListener(onChanged_);
    Proxy.Events.onError.addListener(onChanged_);
  }

  initialize() {
    super.initialize();
    this.onChanged_();
  }

  onChanged_() {
    var errors = [];

    if (!api.isOnLine) {
      errors.push(Errors.TYPE_NO_NETWORK);
    }

    if (offroad.isEnabled) {
      errors.push(Errors.TYPE_OFF_ROAD);
    }

    if (!sessionCookie.isStorable) {
      errors.push(Errors.TYPE_COOKIES);
    }

    if (proxy.levelOfControl === Proxy.CONTROLLABLE_NOT) {
      errors.push(Errors.TYPE_PROXY_SYSTEM);
    } else if (proxy.levelOfControl === Proxy.CONTROLLED_BY_OTHER_EXTENSION) {
      errors.push(Errors.TYPE_PROXY_EXTENSION);
    }

    if (device.isLimitReached) {
      errors.push(Errors.TYPE_DEVICES_LIMIT);
    }

    if (proxy.hasServerErrors) {
      errors.push(Errors.TYPE_PROXY_SERVER);
    }

    if (networkTasks.lastError) {
      errors.push(Errors.TYPE_API);
    }

    if (this.hasUnexpectedError_) {
      errors.push(Errors.TYPE_UNEXPECTED);
    }

    var isDifferent = this.activeErrorsList_.length !== errors.length ||
        (Boolean(errors.length) &&  errors.every(function(element, index) {
              return element === this.activeErrorsList_[index];
            }.bind(this)));

    if (isDifferent) {
      var wasActiveErrorCritical = this.isActiveErrorCritical;
      this.activeErrorsList_ = errors;
      Errors.Events.onChanged.dispatch(this);

      if (!this.isActiveErrorCritical && wasActiveErrorCritical) {
        Errors.Events.onCriticalErrorFixed.dispatch(this);
      }
    }
  }

  reload() {
    this.onChanged_();
  }

  throwUnexpected() {
    if (this.hasUnexpectedError_) {
      return;
    }

    this.hasUnexpectedError_ = true;
    this.onChanged_();
  }


  get hasActive() {
    return Boolean(this.activeErrorsList.length);
  }

  get activeError() {
    return this.activeErrorsList[0];
  }

  get isActiveErrorCritical() {
    return [
      Errors.TYPE_NO_NETWORK,
      Errors.TYPE_OFF_ROAD,
      Errors.TYPE_COOKIES,
      Errors.TYPE_UNEXPECTED,
      Errors.TYPE_PROXY_SYSTEM,
      Errors.TYPE_PROXY_EXTENSION
    ].indexOf(this.activeError) !== -1;
  }

  get activeErrorsList() {
    return this.activeErrorsList_;
  }
};

Errors.TYPE_UNEXPECTED = 'unexpected';
Errors.TYPE_NO_NETWORK = 'no-network';
Errors.TYPE_COOKIES = 'cookies';
Errors.TYPE_OFF_ROAD = 'off-road';
Errors.TYPE_DEVICES_LIMIT = 'device-limit';
Errors.TYPE_PROXY_SYSTEM = 'proxy-system';
Errors.TYPE_PROXY_EXTENSION = 'proxy-extension';
Errors.TYPE_PROXY_SERVER = 'proxy-server';
Errors.TYPE_API = 'api';

Errors.CODE_API_NETWORK = '-1';
Errors.CODE_API_SERVER = '-2';

Errors.Events = {
  onChanged: new chrome.Event(),
  onCriticalErrorFixed: new chrome.Event()
};

window.Errors = Errors;
