'use strict';

class NetworkTask {
  constructor(name, fetcher, canRun) {
    this.data_ = null;
    this.validTill_ = 0;
    this.isRunning_ = false;
    this.name_ = name;
    this.isCacheable_ = true;
    this.fetcher_ = fetcher;
    this.canRun_ = canRun;
  }

  get name() {
    return this.name_;
  }

  get isValid() {
    return this.validTill_ > Date.now();
  }

  get isRunning() {
    return this.isRunning_;
  }

  get canRun() {
    return !errors.isActiveErrorCritical && this.canRun_();
  }

  get isCacheable() {
    return this.isCacheable_;
  }

  set isCacheable(value) {
    this.isCacheable_ = value;

    if (this.isCacheable) {
      settings.set(NetworkTask.ID + '.' + this.name, this.data);
    } else {
      settings.remove(NetworkTask.ID + '.' + this.name);
    }
  }

  get data() {
    return this.data_ !== null ? this.data_ : (this.isCacheable ?
        settings.get(NetworkTask.ID + '.' + this.name) : null);
  }

  set data(value) {
    if (JSON.stringify(value) === JSON.stringify(this.data_)) {
      return;
    }

    this.data_ = value;

    if (this.isCacheable && value !== null) {
      settings.set(NetworkTask.ID + '.' + this.name, value);
    }

    NetworkTask.Events.onData.dispatch(this);
  }

  clear() {
    this.validTill_ = 0;
    this.data = null;

    if (this.isCacheable) {
      settings.set(NetworkTask.ID + '.' + this.name, null);
    }

    this.onRunCompleted_();
  }

  run() {
    if (this.isRunning) {
      return this.runPromise_;
    }

    if (!this.canRun) {
      return Promise.reject();
    }

    this.isRunning_ = true;

    this.runPromise_ = new Promise(function(resolve, reject) {
      this.runPromiseResolve_ = resolve;
      (new Promise(this.fetcher_)).then(function(networkTaskData) {
        if (!this.canRun) {
          reject();
        }

        this.validTill_ = networkTaskData.validTill;
        this.data = networkTaskData.data;
        this.onRunCompleted_();
      }.bind(this)).catch(function(details) {
        if (!this.isRunning_) {
          return;
        }

        if (!this.canRun) {
          reject();
        }

        this.isRunning_ = false;
        this.data = null;
        reject(details);
      }.bind(this));
    }.bind(this));

    this.runPromise_.catch(function(details) {
      if (!details) {
        return;
      }
      NetworkTask.Events.onFail.dispatch({
        target: this,
        details: details
      });
    }.bind(this));

    return this.runPromise_;
  }

  onRunCompleted_() {
    if (!this.isRunning_) {
      return;
    }

    this.isRunning_ = false;

    if (this.runPromiseResolve_) {
      this.runPromiseResolve_();
      this.runPromiseResolve_ = null;
    }
  }
}

NetworkTask.Events = {
  onData: new chrome.Event(),
  onFail: new chrome.Event()
};

NetworkTask.ID = 'networkTask';

window.NetworkTask = NetworkTask;
