'use strict';

class Notification {
  constructor(data) {
    this.data_ = data;

    this.isVisible_ = false;

    this.hideTimeout_ = null;

    chrome.notifications.onClicked.addListener(this.onClicked_.bind(this));
    chrome.notifications.onClosed.addListener(this.onClosed_.bind(this));
  }

  get isVisible() {
    return this.isVisible_;
  }

  set isVisible(value) {
    if (this.isVisible_ === value) {
      return;
    }

    chrome.notifications.clear(this.group);
    this.isVisible_ = value;

    if (this.hideTimeout_) {
      clearTimeout(this.hideTimeout_);
      this.hideTimeout_ = null;
    }

    if (this.isVisible_) {
      if (app.isPopupVisible) {
        this.hideTimeout_ = setTimeout(function() {
          this.isVisible = false;
        }.bind(this), Notification.HIDE_TIMEOUT);
      } else {
        chrome.notifications.create(this.group, this.data);
      }
    }

    Notification.Events.onChanged.dispatch(this);
  }

  get group() {
    return this.data_.group;
  }

  set group(value) {
    this.data_.group = value;
  }

  get id() {
    return this.data_.id;
  }

  set id(value) {
    this.data_.id = value;
  }

  get type() {
    return this.data_.type || Notification.TYPE_BASIC;
  }

  set type(value) {
    this.data_.type = value;
  }

  get title() {
    return this.data_.title;
  }

  set title(value) {
    this.data_.title = value;
  }

  get message() {
    return this.data_.message;
  }

  set message(value) {
    this.data_.message = value;
  }

  get data() {
    return {
      'type': this.type,
      'title': this.title,
      'message': this.message,
      'iconUrl': Notification.ICON
    };
  }

  onClicked_(group) {
    if (group === this.group && this.isVisible) {
      Notification.Events.onClicked.dispatch(this);
      this.isVisible = false;
    }
  }

  onClosed_(group) {
    if (group === this.group && this.isVisible) {
      this.isVisible = false;
    }
  }
};

Notification.Events = {
  onClicked: new chrome.Event(),
  onChanged: new chrome.Event()
};

Notification.TYPE_BASIC = 'basic';

Notification.ID_CONNECTED = 0;
Notification.ID_USED = 1;
Notification.ID_OVER_BANDWIDTH = 2;
Notification.ID_EXPIRED = 3;

Notification.ICON = Url.IMAGES + 'notification.png';
Notification.HIDE_TIMEOUT = 2000;

window.Notification = Notification;
