'use strict';

class OffRoad extends Base {
  constructor() {
    super();
    this.isSupported_ = App.browserType === App.BROWSER_TYPE_OPERA &&
        window.opr && window.opr.offroad;
    this.isEnabled_ = false;
    this.levelOfControl_ = null;
  }

  registerListeners() {
    super.registerListeners();

    window.opr.offroad.enabled.onChange.addListener(this.onChanged_.bind(this));
  }

  initialize() {
    if (!this.isSupported) {
      return;
    }

    super.initialize();
    this.onChanged_();
  }

  get isSupported() {
    return this.isSupported_;
  }

  get levelOfControl() {
    return this.levelOfControl_;
  }

  get isControllable() {
    return this.levelOfControl === OffRoad.CONTROLLED_BY_OTHER_EXTENSION ||
        this.levelOfControl === OffRoad.CONTROLLABLE_BY_THIS_EXTENSION;
  }

  get isEnabled() {
    return this.isEnabled_;
  }

  set isEnabled(value) {
    if (this.isEnabled === value || !this.isSupported) {
      return;
    }

    window.opr.offroad.enabled.set({'value': value});
  }

  onChanged_() {
    window.opr.offroad.enabled.get({}, function(details) {
      var isEnabled = Boolean(details.value);

      if (details.levelOfControl === this.levelOfControl_ &&
          isEnabled === this.isEnabled) {
        return;
      }

      this.levelOfControl_ = details.levelOfControl;
      this.isEnabled_ = isEnabled;
      OffRoad.Events.onChanged.dispatch(this);
    }.bind(this));
  }
}

OffRoad.CONTROLLABLE_NOT = 'not_controllable';
OffRoad.CONTROLLED_BY_OTHER_EXTENSION = 'controlled_by_other_extensions';
OffRoad.CONTROLLABLE_BY_THIS_EXTENSION = 'controllable_by_this_extension';
OffRoad.CONTROLLED_BY_THIS_EXTENSION = 'controlled_by_this_extension';

OffRoad.Events = {
  onChanged: new chrome.Event()
};

window.OffRoad = OffRoad;