'use strict';

class Reward {
  constructor(data) {
    this.data_ = data;
    this.data_.isLocal = this.data_.isLocal || false;

    if (this.isLocal) {
      settings.register(this.code, this.data_.completed);
    }

    Reward.Events.onChanged.dispatch(this);
  }

  get isLocal() {
    return this.data_.isLocal;
  }

  get amount() {
    return this.data_.amount;
  }

  get code() {
    return this.data_.code;
  }

  get isCompleted() {
    if (this.isLocal) {
      return settings.get(this.code);
    } else {
      return this.data_.completed;
    }
  }

  set isCompleted(value) {
    if (this.isCompleted === value) {
      return;
    }

    var onSaved = new Promise(function(resolve, reject) {
      if (this.isLocal) {
        settings.set(this.code, value);
        resolve();
      } else if (value) {
        api.rewardMarkAsCompleted(this.code).then(resolve).catch(reject);
      } else {
        resolve();
      }
    }.bind(this));

    onSaved.then(function() {
      this.data_.completed = value;
      Reward.Events.onChanged.dispatch(this);
    }.bind(this)).catch(doNothing);
  }

  get description() {
    return this.data_.menu_title;
  }

  get icon() {
    return Reward.SUPPORTED_ICONS.indexOf(this.data_.icon) !== -1 ?
        this.data_.icon : Reward.ICON_DEFAULT;
  }

  get url() {
    return this.data_.url;
  }
}

Reward.Events = {
  onChanged: new chrome.Event()
};

Reward.REWARD_AMOUNT_UNIT = 'MB';

Reward.REWARD_WELCOME = 'reward.welcome';
Reward.REWARD_WELCOME_AMOUNT = '500';
Reward.REWARD_NUMBER_INPUT = 'reward.numberInput';
Reward.REWARD_NUMBER_INPUT_AMOUNT = '250';

Reward.ICON_DEFAULT = 'icon-default';

Reward.SUPPORTED_ICONS = [
  'icon-envelope-alt',
  'icon-heart',
  'icon-map-marker',
  'icon-plus-sign',
  'icon-thumbs-up-alt',
  'icon-time',
  'icon-twitter-sign',
  'icon-fb-sign',
  'icon-star'
];

window.Reward = Reward;
