'use strict';

class Url {
  static open(url) {
    chrome.tabs.create({
      'url': url
    });
  }

  static openOrSelect(url) {
    chrome.tabs.query({
          'url': url
        }, function (tabs) {
      if (!tabs || !tabs.length) {
        Url.open(url);
      } else {
        var hasActiveTab = tabs.some(function(tab) {
          return tab.active;
        });

        if (!hasActiveTab) {
          chrome.tabs.update(tabs[0].id, {
            'active': true
          });
        }
      }
    });
  }

  static replace(url, newURL) {
    var params = {
      'active': true,
      'url': newURL
    };

    chrome.tabs.query({}, function (tabs) {
      if (!tabs || !tabs.length) {
        chrome.tabs.create(params);
      }

      tabs = tabs.filter(function(tab) {
        return tab.url.startsWith(url);
      });

      if (!tabs.length) {
        chrome.tabs.create(params);
      } else if (tabs.length === 1) {
        chrome.tabs.update(tabs[0].id, params);
      } else {
        var hasActiveTab = tabs.some(function(tab) {
          return tab.active;
        });

        if (hasActiveTab) {
          tabs.forEach(function(tab) {
            if (tab.active) {
              chrome.tabs.update(tab.id, params);
            } else {
              chrome.tabs.remove(tab.id);
            }
          });
        } else {
          var tab = tabs.pop();
          chrome.tabs.update(tab.id, params);
          tabs.forEach(function(tab) {
            chrome.tabs.remove(tab.id);
          });
        }
      }
    });
  }

  static close(url) {
    chrome.tabs.query({}, function (tabs) {
      if (tabs && tabs.length) {
        if (tabs.length === 1) {
          chrome.tabs.create({'active': true});
        }

        var tabsToClose = tabs.filter(function(tab) {
          return tab.url === url;
        });

        tabsToClose.forEach(function(tab) {
          chrome.tabs.remove(tab.id);
        });
      }
    });
  }

  static openAutorized(url) {
    user.getAutoLoginToken().then(function(token) {
      url = Url.addParam(url, Url.PARAM_TOKEN, token);
      Url.open(url);
    }).catch(Url.open.bind(Url, url));
  }

  static withStagingPostfix(str) {
    return str + (Api.isUsingStaging ? Url.STAGING_POSTFIX : '');
  }

  static addParam(url, name, value) {
    url = new URL(url);
    url.search = url.search + (url.search.length ? '&' : '') + name + '=' +
        value;
    return url.href;
  }

  static hasParam(url, name) {
    url = new URL(url);
    return url.search.length &&
        (new RegExp('[?&]+'+ name +'=','gi')).test(url.search);
  }

  static openMiniInternetTab(deviceId, url) {
    var newURL = url;

    if (deviceId) {
      newURL = Url.addParam(newURL, Url.PARAM_DEVICE_ID, hash(device.id));
    }

    user.getAutoLoginToken().then(function(token) {
      newURL = Url.addParam(newURL, Url.PARAM_TOKEN, token);
      Url.replace(url, newURL);
    }).catch(function() {
      Url.replace(url, newURL);
    });
  }
};

Url.STAGING_POSTFIX = '-staging';

Url.HOST = 'surfeasy.com';
Url.HOST_PATTERN = '*.' + Url.HOST;
Url.HTTPS = 'https://';
Url.HTTP = 'http://';
Url.API = Url.HTTPS + Url.withStagingPostfix('api') + '.' + Url.HOST;
Url.BASE = Url.HTTPS + Url.HOST;
Url.USER_DASHBOARD = Url.HTTPS + Url.withStagingPostfix('accounts') + '.' +
    Url.HOST;
Url.USER_RESTORE_PASSWORD = Url.USER_DASHBOARD + '/lost_password';
Url.PLAN_OVER_BANDWIDTH = Url.USER_DASHBOARD + '/over_bandwidth';
Url.PLAN_EXPIRED = Url.USER_DASHBOARD + '/expired';
Url.TOOLBAR_POPUP = '/pages/popup/popup.html';
Url.LANDING_PAGE = '/pages/landing/landing.html';
Url.PURCHASE_PAGE = Url.USER_DASHBOARD + '/upgrade';

Url.PROXY_SERVER_PATTERN_DEFAULT = '%@.se-proxy.net';
Url.PROXY_PAC_SCRIPT = '/pages/background/js/pac_script.js';

Url.REWARDS_PAGE = Url.USER_DASHBOARD + '/rewards';
Url.IMAGES = '/images/';
Url.REWARDS_ICON_SRC = Url.IMAGES + 'rewards/$1.svg';

Url.ABOUT_US = Url.BASE + '/about-us/';
Url.TERMS_OF_SERVICE = Url.BASE + '/terms_of_service/';
Url.PRIVACY_POLICY = Url.BASE + '/privacy_policy/';
Url.SUPPORT = Url.HTTPS + 'support.' + Url.HOST + '/hc/categories/200360433';
Url.SUPPORT_EMAIL = 'support@' + Url.HOST;

Url.PARAM_TOKEN = 'alt';
Url.PARAM_DEVICE_ID = 'data';

Url.UNINSTALL = 'https://www.surveymonkey.com/r/surfeasy-uninstall';

Url.UNKNOWN_HOST = 'http://home.surfeasy.com/?q=';

window.Url = Url;