'use strict';

class DevView extends AbstractView {
  constructor() {
    super(DevView.ID);

    this.title_ = 'Developer options';
    this.canShow_ = Boolean(this.stagingApiKey);

    this.registerListeners_();
  }

  get stagingApiKey() {
    return window.location.hash.substring(1);
  }

  get items() {
    return [
      {
        'label': i18n.get('buttonReset'),
        'data-handler': DevView.HANDLER_RESET
      },
      {
        'label': 'Check for updates',
        'data-handler': DevView.HANDLER_CHECK_UPDATE
      },
      {
        'label': 'Generate report',
        'data-handler': DevView.HANDLER_REPORT
      },
      {
        'label': 'Use ' + (Api.isUsingStaging ? 'production' : 'staging'),
        'data-handler': DevView.HANDLER_TOGGLE_STAGING
      }
    ];
  }

  render() {
    renderTemplateIfNeeded(this.viewElement, this.Templates.view(this.items));
    renderTemplateIfNeeded(this.headerElement,
        this.Templates.header(Api.isUsingStaging));
  }

  registerHandlers_() {
    super.registerHandlers_();

    EventHandler.register('click', DevView.HANDLER_RESET,
                          bindOnce(this.onReset_, this));
    EventHandler.register('click', DevView.HANDLER_CHECK_UPDATE,
                          bindOnce(this.onCheckUpdate_, this));
    EventHandler.register('click', DevView.HANDLER_REPORT,
                          bindOnce(this.onReport_, this));
    EventHandler.register('click', DevView.HANDLER_TOGGLE_STAGING,
                          bindOnce(this.onToggleStaging_, this));
  }

  unregisterHandlers_() {
    super.unregisterHandlers_();

    EventHandler.unregister('click', DevView.HANDLER_RESET,
                            bindOnce(this.onReset_, this));
    EventHandler.unregister('click', DevView.HANDLER_CHECK_UPDATE,
                            bindOnce(this.onCheckUpdate_, this));
    EventHandler.unregister('click', DevView.HANDLER_REPORT,
                            bindOnce(this.onReport_, this));
    EventHandler.unregister('click', DevView.HANDLER_TOGGLE_STAGING,
                            bindOnce(this.onToggleStaging_, this));
  }

  onReset_() {
    if (Api.isUsingStaging) {
      user.logout();
      localStorage.clear();
      settings.set(Api.SETTING_CLIENT_API_KEY, this.stagingApiKey);
      app.restart();
    } else {
      app.reset();
    }
  }

  onCheckUpdate_() {
    chrome.runtime.requestUpdateCheck(function(status, details) {
      if (status !== 'update_available') {
        alert('You are using the latest version');
      }
    });
  }

  onReport_() {
    app.report();
  }

  onToggleStaging_() {
    if (!Api.isUsingStaging) {
      user.logout();
      localStorage.clear();
      settings.set(Api.SETTING_CLIENT_API_KEY, this.stagingApiKey);
      app.restart();
    } else {
      app.reset();
    }
  }

};

DevView.Templates = class {
  static header(isUsingStaging) {
    return [
      'h1',
      ['strong', 'Developer options'],
      ['span', 'You are using ' + (isUsingStaging ? 'staging' : 'production')]
    ];
  }

  static view(list) {
    return [
      [
        'div', {
          'class': 'list'
        },
        list.map(function(data) {
          data.class = [
            'list-item',
            data.class ? data.class : 'is-link'
          ].join(' ').trim();
          var label = data.label;
          delete data.label;

          return ['div', data, label];
        })
      ]
    ];
  }
};

DevView.ID = 'dev';

DevView.HANDLER_RESET = 'dev-reset';
DevView.HANDLER_CHECK_UPDATE = 'dev-check-update';
DevView.HANDLER_REPORT = 'dev-report';
DevView.HANDLER_TOGGLE_STAGING = 'dev-toggle-staging';