'use strict';

class AbstractPage {
  constructor(id) {
    this.id_ = id;
    this.title_ = '';
    this.viewElement_ = bragi.createTemplate([
      'div',
      {'id': this.id_ + '-page'}
    ]);
  }

  get title() {
    return this.title_;
  }

  set title(value) {
    if (this.title != value) {
      this.title_ = value;
      AbstractPage.Events.onTitleChanged.dispatch(this);
    }
  }

  get Templates() {
    return this.constructor.Templates;
  }

  get id() {
    return this.id_;
  }

  get viewElement() {
    return this.viewElement_;
  }

  get isVisible() {
    return Boolean(this.viewElement.parentNode);
  }

  show(container) {
    if (!this.isVisible) {
      this.render();
      this.registerHandlers_();
    }
    container.appendChild(this.viewElement);
    AbstractPage.Events.onVisibilityChanged.dispatch(this);
  }

  hide() {
    if (this.isVisible) {
      this.unregisterHandlers_();
      this.clean();
      AbstractPage.Events.onVisibilityChanged.dispatch(this);
    }
  }

  render() {}

  clean() {
    this.viewElement.remove();
    this.viewElement.textContent = '';
  }

  registerHandlers_() {}
  unregisterHandlers_() {}

  registerListeners_() {
    window.addEventListener('unload',
        this.unregisterListeners_.bind(this), true);
  }
  unregisterListeners_() {}
};

AbstractPage.Events = {
  onVisibilityChanged: new chrome.Event(),
  onTitleChanged: new chrome.Event()
};
