'use strict';

class AbstractSite {
  constructor(id) {
    this.id_ = id;
    this.siteElement_ = null;
    this.pages_ = [];
    this.views_ = [];
  }

  get id() {
    return this.id_;
  }

  get title() {
    return document.title;
  }

  set title(value) {
    if (!value) {
      value = i18n.get('extensionName');
    }
    document.title = value;
  }

  get Templates() {
    return this.constructor.Templates;
  }

  get siteElement() {
    return this.siteElement_;
  }

  get headerElement() {
    return this.siteElement.querySelector(idToSelector(AbstractSite.ID_HEADER));
  }

  get contentElement() {
    return this.siteElement.querySelector(idToSelector(
        AbstractSite.ID_CONTENT));
  }

  get footerElement() {
    return this.siteElement.querySelector(idToSelector(AbstractSite.ID_FOOTER));
  }

  addPage(page) {
    this.pages_.push(page);
  }

  getPage(id) {
    return this.pages_.filter(function(page) {
      return page.id === id;
    })[0];
  }

  addView(view) {
    this.views_.push(view);
  }

  getView(id) {
    return this.views_.filter(function(view) {
      return view.id === id;
    })[0];
  }

  get currentView() {
    return this.views_.filter(function(view) {
      return view.isVisible;
    })[0];
  }

  initialize() {
    this.registerListeners_();
    document.dir = i18n.dir;
    this.render();
    setTimeout(this.onCanShowChanged_.bind(this), 0);
  }

  render() {
    this.siteElement_ = document.body.appendTemplate(this.Templates.view(
        this.id));
  }

  showNextView() {
    var next = this.views_.filter(function(view) {
      return view.canShow;
    })[0];

    if (this.currentView === next) {
      return;
    }

    if (this.currentView) {
      this.currentView.viewElement.remove();
      this.currentView.headerElement.remove();
      this.currentView.footerElement.remove();
      this.currentView.isVisible = false;
    }

    if (next) {
      next.isVisible = true;
      this.contentElement.appendChild(next.viewElement);
      this.headerElement.appendChild(next.headerElement);
      this.footerElement.appendChild(next.footerElement);
      this.title = next.title;
    }
  }

  onCanShowChanged_() {
    this.showNextView();
  }

  onPageViewChanged_(view) {
    if (!view || !view.isVisible) {
      return;
    }

    var pages = [this.id, view.id];

    if (view.id === ErrorView.ID) {
      if (view.errorType) {
        pages.push(view.errorType);
        analytics.countPage(pages);
      }
    } else {
      if (view.currentPageID) {
        pages.push(view.currentPageID);
      }

      analytics.countPage(pages);
    }
  }

  onViewChanged_(view) {
    if (!view || !view.isVisible) {
      return;
    }

    var pages = [this.id, view.id];

    if (view.id === ErrorView.ID) {
      if (view.errorType) {
        pages.push(view.errorType);
        analytics.countPage(pages);
      }
    } else if (view instanceof AbstractPageView) {
      if (view.defaultPageID === null) {
        analytics.countPage(pages);
      }
    } else {
      analytics.countPage(pages);
    }
  }

  onTitleChanged_() {
    this.title = this.currentView ? this.currentView.title : '';
  }

  registerListeners_() {
    AbstractView.Events.onCanShowChanged.addListener(
        this.onCanShowChanged_.bind(this));
    AbstractView.Events.onTitleChanged.addListener(
        this.onTitleChanged_.bind(this));
    AbstractView.Events.onVisibilityChanged.addListener(
        this.onViewChanged_.bind(this));
    AbstractPageView.Events.onPageChanged.addListener(
        this.onPageViewChanged_.bind(this));
  }
};

AbstractSite.Templates = class {
  static footer() {
    return ['footer', {'id': AbstractSite.ID_FOOTER}];
  }

  static header() {
    return ['header', {'id': AbstractSite.ID_HEADER}];
  }

  static content() {
    return ['section', {'id': AbstractSite.ID_CONTENT}];
  }

  static view(id) {
    return [
      'div', {
        'id': id
      },
      this.header(),
      this.content(),
      this.footer()
    ];
  }
};

AbstractSite.ID = 'site';
AbstractSite.ID_HEADER = 'site-header';
AbstractSite.ID_CONTENT = 'site-content';
AbstractSite.ID_FOOTER = 'site-footer';