'use strict';

class AbstractView {
  constructor(id) {
    this.id_ = id;
    this.canShow_ = false;
    this.isVisible_ = false;
    this.title_ = '';
    this.viewElement_ = bragi.createTemplate(
        ['div', {'id': this.id_ + '-view'}]);
    this.headerElement_ = bragi.createTemplate(
        ['div', {'id': this.id_ + '-header'}]);
    this.footerElement_ = bragi.createTemplate(
        ['div', {'id': this.id_ + '-footer'}]);

    this.headerType = AbstractView.HEADER_DEFAULT;
  }

  get id() {
    return this.id_;
  }

  get title() {
    return this.title_;
  }

  set title(value) {
    if (this.title != value) {
      this.title_ = value;
      AbstractView.Events.onTitleChanged.dispatch(this);
    }
  }

  get Templates() {
    return this.constructor.Templates;
  }

  get viewElement() {
    return this.viewElement_;
  }

  get headerElement() {
    return this.headerElement_;
  }

  get footerElement() {
    return this.footerElement_;
  }

  get headerType() {
    if (this.headerElement.classList.contains(AbstractView.HEADER_DEFAULT)) {
      return AbstractView.HEADER_DEFAULT;
    }

    if (this.headerElement.classList.contains(AbstractView.HEADER_NORMAL)) {
      return AbstractView.HEADER_NORMAL;
    }

    if (this.headerElement.classList.contains(AbstractView.HEADER_ERORR)) {
      return AbstractView.HEADER_ERORR;
    }
  }

  set headerType(value) {
    this.headerElement.classList.toggle(AbstractView.HEADER_DEFAULT,
        value === AbstractView.HEADER_DEFAULT);
    this.headerElement.classList.toggle(AbstractView.HEADER_NORMAL,
        value === AbstractView.HEADER_NORMAL);
    this.headerElement.classList.toggle(AbstractView.HEADER_ERORR,
        value === AbstractView.HEADER_ERORR);
  }

  get isVisible() {
    return this.isVisible_;
  }

  set isVisible(value) {
    if (this.isVisible_ === value) {
      return;
    }

    this.isVisible_ = value;
    if (this.isVisible_) {
      this.render();
      this.registerHandlers_();
    } else {
      this.clean();
      this.unregisterHandlers_();
    }

    AbstractView.Events.onVisibilityChanged.dispatch(this);
  }

  set canShow(value) {
    if (this.canShow_ === value) {
      return;
    }
    this.canShow_ = value;
    AbstractView.Events.onCanShowChanged.dispatch(this);
  }

  get canShow() {
    return this.canShow_;
  }

  clean() {
    this.viewElement.textContent = '';
    this.headerElement.textContent = '';
    this.footerElement.textContent = '';
  }

  render() {}

  registerHandlers_() {
    AbstractPage.Events.onTitleChanged.addListener(
        bindOnce(this.onTitleChanged_, this));
  }
  unregisterHandlers_() {
    AbstractPage.Events.onTitleChanged.removeListener(
        bindOnce(this.onTitleChanged_, this));
  }

  registerListeners_() {
    window.addEventListener('unload',
        this.unregisterListeners_.bind(this), true);
  }
  unregisterListeners_() {}

  onTitleChanged_() {
    AbstractView.Events.onTitleChanged.dispatch(this);
  }
};

AbstractView.Events = {
  onVisibilityChanged: new chrome.Event(),
  onCanShowChanged: new chrome.Event(),
  onTitleChanged: new chrome.Event()
};

AbstractView.HEADER_DEFAULT = 'default';
AbstractView.HEADER_NORMAL = 'normal';
AbstractView.HEADER_ERORR = 'error';
