'use strict';

class ErrorView extends AbstractPageView {
  constructor() {
    super(ErrorView.ID);
    this.headerType = AbstractView.HEADER_NORMAL;

    this.registerListeners_();
    this.onErrorChanged_();
  }

  get errorType() {
    return this.errorType_;
  }

  set errorType(value) {
    if (this.errorType_ === value) {
      return;
    }

    this.errorType_ = value;

    if (this.isVisible && Boolean(this.errorType_)) {
      this.render();
    }
  }

  render() {
    if (!this.isPageVisible) {
      renderTemplateIfNeeded(this.viewElement,
          this.Templates.view(this.errorType));
    } else {
      this.renderSubpage(this.Templates.page());
    }

    this.headerElement.classList.toggle(AbstractPageView.CLASS_IS_PAGE_VISIBLE,
        this.isPageVisible);
    renderTemplateIfNeeded(this.headerElement,
        this.Templates.header(this.isPageVisible));
  }

  registerHandlers_() {
    super.registerHandlers_();
    EventHandler.register('click', ErrorView.HANDLER_OFF_ROAD_OFF,
                          bindOnce(this.onTurnOffRoadOff_, this));
    EventHandler.register('click', ErrorView.HANDLER_COOKIES,
                          bindOnce(this.onEnableCookies_, this));
    EventHandler.register('click', ErrorView.HANDLER_RESET,
                          bindOnce(this.onReset_, this));
    EventHandler.register('click', ErrorView.HANDLER_RESTART,
                          bindOnce(this.onRestart_, this));
    EventHandler.register('click', ErrorView.HANDLER_RUN_TASKS_AGAIN,
                          bindOnce(this.onRunTasksAgain_, this));
    EventHandler.register('click', ErrorView.HANDLER_MANAGE_ACCOUNT,
                          bindOnce(this.onManageAccount_, this));
    EventHandler.register('click', ErrorView.HANDLER_CHECK_DEVICE_LIMIT,
                          bindOnce(this.onCheckDeviceLimit_, this));
    EventHandler.register('click', ErrorView.HANDLER_UNINSTALL_EXTENSIONS,
                          bindOnce(this.onUninstallOtherProxyExtensions_,
                                   this));
    LocationPage.Events.onSelected.addListener(
        bindOnce(this.onLocationSelected_, this));
  }

  unregisterHandlers_() {
    super.unregisterHandlers_();
    EventHandler.unregister('click', ErrorView.HANDLER_OFF_ROAD_OFF,
                            bindOnce(this.onTurnOffRoadOff_, this));
    EventHandler.unregister('click', ErrorView.HANDLER_COOKIES,
                            bindOnce(this.onEnableCookies_, this));
    EventHandler.unregister('click', ErrorView.HANDLER_RESET,
                            bindOnce(this.onReset_, this));
    EventHandler.unregister('click', ErrorView.HANDLER_RESTART,
                            bindOnce(this.onRestart_, this));
    EventHandler.unregister('click', ErrorView.HANDLER_RUN_TASKS_AGAIN,
                            bindOnce(this.onRunTasksAgain_, this));
    EventHandler.unregister('click', ErrorView.HANDLER_MANAGE_ACCOUNT,
                            bindOnce(this.onManageAccount_, this));
    EventHandler.unregister('click', ErrorView.HANDLER_CHECK_DEVICE_LIMIT,
                            bindOnce(this.onCheckDeviceLimit_, this));
    EventHandler.unregister('click', ErrorView.HANDLER_UNINSTALL_EXTENSIONS,
                            bindOnce(this.onUninstallOtherProxyExtensions_,
                                     this));
    LocationPage.Events.onSelected.removeListener(
        bindOnce(this.onLocationSelected_, this));
  }

  registerListeners_() {
    super.registerListeners_();
    Errors.Events.onChanged.addListener(bindOnce(this.onErrorChanged_, this));
  }

  unregisterListeners_() {
    super.unregisterListeners_();
    Errors.Events.onChanged.removeListener(
        bindOnce(this.onErrorChanged_, this));
  }

  onEnableCookies_() {
    sessionCookie.addException();
  }

  onManageAccount_() {
    Url.openAutorized(Url.USER_DASHBOARD);
  }

  onUninstallOtherProxyExtensions_() {
    app.disableProxyExtensions();
  }

  onCheckDeviceLimit_() {
    device.checkLimit();
  }

  onReset_() {
    app.reset();
  }

  onRestart_() {
    app.restart();
  }

  onRunTasksAgain_() {
    networkTasks.runAgain();
  }

  onTurnOffRoadOff_() {
    offroad.isEnabled = false;
  }

  onLocationSelected_(code) {
    geo.countryCode = code;
    this.currentPageID = this.defaultPageID;
    app.restart();
  }

  onErrorChanged_() {
    this.errorType = errors.activeError;
    this.canShow = errors.hasActive;
  }
};

ErrorView.Templates = class extends AbstractPageView.Templates {
  static errorHeader() {
    return [
      'header', {
        'class': 'error-icon'
      }
    ];
  }

  static errorMessage(message) {
    return ['p', i18n.get('errorTitle', [message])];
  }

  static contentUnexpected() {
    return [
      this.errorMessage(i18n.get('errorMessageUnexpected')),
      [
        'button', {
          'class': 'error',
          'data-handler': ErrorView.HANDLER_RESTART
        },
        i18n.get('buttonTryAgain')
      ],
      [
        'button', {
          'class': 'background',
          'data-handler': ErrorView.HANDLER_RESET
        },
        i18n.get('buttonReset')
      ]
    ];
  }

  static contentProxyServer() {
    return [
      this.errorMessage(i18n.get('errorMessageProxyServer')),
      [
        'button', {
          'class': 'error',
          'data-handler': AbstractPageView.HANDLER_NAVIGATE_TO_PAGE,
          'data-page': LocationPage.ID
        },
        i18n.get('buttonSelectDifferentRegion')
      ],
      [
        'button', {
          'class': 'background',
          'data-handler': ErrorView.HANDLER_RESTART
        },
        i18n.get('buttonTryAgain')
      ]
    ];
  }

  static contentProxyExtension() {
    return [
      this.errorMessage(i18n.get('errorMessageProxyExtension')),
      [
        'button', {
          'class': 'error',
          'data-handler': ErrorView.HANDLER_UNINSTALL_EXTENSIONS
        },
        i18n.get('buttonDisableOtherProxyExtension')
      ]
    ];
  }

  static contentApi(message) {
    return [
      this.errorMessage(message),
      [
        'button', {
          'class': 'error',
          'data-handler': ErrorView.HANDLER_RUN_TASKS_AGAIN
        },
        i18n.get('buttonTryAgain')
      ]
    ];
  }

  static contentDeviceLimit() {
    return [
      this.errorMessage(i18n.get('errorMessageDeviceLimit')),
      [
        'button', {
          'class': 'error',
          'data-handler': ErrorView.HANDLER_MANAGE_ACCOUNT
        },
        i18n.get('buttonManageAccount')
      ],
      [
        'button', {
          'class': 'background',
          'data-handler': ErrorView.HANDLER_CHECK_DEVICE_LIMIT
        },
        i18n.get('buttonTryAgain')
      ]
    ];
  }

  static contentOffRoad() {
    return [
      this.errorMessage(i18n.get('errorMessageOffRoad')),
      [
        'button', {
          'class': 'error',
          'data-handler': ErrorView.HANDLER_OFF_ROAD_OFF
        },
        i18n.get('buttonTurnOffRoadOff')
      ]
    ];
  }

  static contentCookies() {
    return [
      this.errorMessage(i18n.get('errorMessageCookies')),
      [
        'button', {
        'class': 'error',
        'data-handler': ErrorView.HANDLER_COOKIES
      },
        i18n.get('buttonEnableCookies')
      ]
    ];
  }

  static view(errorType) {
    if (errorType === undefined) {
      return [];
    }

    var content = this.contentUnexpected();

    if (errorType === Errors.TYPE_NO_NETWORK) {
      content = this.errorMessage(i18n.get('errorMessageNoNetwork'));
    } else if (errorType === Errors.TYPE_OFF_ROAD) {
      content = this.contentOffRoad();
    } else if (errorType === Errors.TYPE_COOKIES) {
      content = this.contentCookies();
    } else if (errorType === Errors.TYPE_PROXY_SERVER) {
      content = this.contentProxyServer();
    } else if (errorType === Errors.TYPE_API) {
      content = this.contentApi(networkTasks.lastError);
    } else if (errorType === Errors.TYPE_DEVICES_LIMIT) {
      content = this.contentDeviceLimit();
    } else if (errorType === Errors.TYPE_PROXY_SYSTEM) {
      content = this.errorMessage(i18n.get('errorMessageProxySystem'));
    } else if (errorType === Errors.TYPE_PROXY_EXTENSION) {
      content = this.contentProxyExtension();
    }

    return [
      this.errorHeader(),
      content
    ];
  }
};

ErrorView.ID = 'error';

ErrorView.HANDLER_RESET = 'error-reset';
ErrorView.HANDLER_RESTART = 'error-restart';
ErrorView.HANDLER_RUN_TASKS_AGAIN = 'error-run-tasks-again';
ErrorView.HANDLER_MANAGE_ACCOUNT = 'error-manage-account';
ErrorView.HANDLER_OFF_ROAD_OFF = 'error-off-road';
ErrorView.HANDLER_COOKIES = 'error-cookies';
ErrorView.HANDLER_CHECK_DEVICE_LIMIT = 'error-check-device-limit';
ErrorView.HANDLER_UNINSTALL_EXTENSIONS = 'error-uninstall-extensions';
