'use strict';

class LoadingView extends AbstractPageView {
  constructor() {
    super(LoadingView.ID);
    this.title_ = i18n.get('loadingTitle');
    this.headerType = AbstractView.HEADER_NORMAL;

    this.registerListeners_();
    this.onCanShowChanged_();
  }

  registerListeners_() {
    super.registerListeners_();
    var onCanShowChanged_ = bindOnce(this.onCanShowChanged_, this);
    ToolbarState.Events.onChanged.addListener(onCanShowChanged_);
    Proxy.Events.onCanConnectChanged.addListener(onCanShowChanged_);
  }

  unregisterListeners_() {
    super.unregisterListeners_();
    var onCanShowChanged_ = bindOnce(this.onCanShowChanged_, this);
    ToolbarState.Events.onChanged.removeListener(onCanShowChanged_);
    Proxy.Events.onCanConnectChanged.removeListener(onCanShowChanged_);
  }

  render() {
    if (!this.isPageVisible) {
      renderTemplateIfNeeded(this.viewElement, this.Templates.view());
      this.fillMessage_();
    } else {
      this.renderSubpage(this.Templates.page());
    }

    this.headerElement.classList.toggle(AbstractPageView.CLASS_IS_PAGE_VISIBLE,
        this.isPageVisible);
    renderTemplateIfNeeded(this.headerElement,
        this.Templates.header(this.isPageVisible));
  }

  fillMessage_() {
    var message = i18n.get(proxy.canConnect ? 'loadingSecuringTitle' :
        'loadingTitle');
    var element = this.viewElement.querySelector(idToSelector(
        LoadingView.ID_MESSAGE));

    element.textContent = message;
  }

  onCanShowChanged_() {
    var canShow = toolbarState.state === ToolbarState.STATE_CONNECTING;
    if (this.canShow !== canShow) {
      this.canShow = canShow;
    } else if (this.isVisible && !this.isPageVisible) {
      this.fillMessage_();
    }
  }
};

LoadingView.Templates = class extends AbstractPageView.Templates {
  static view() {
    return [
      [
        'header', {
          'class': 'loading-spinner'
        },
        ['div', {'class': 'spinner-shadow'}],
        ['div', {'class': 'spinner'}],
        ['div', {'class': 'spinner-shadow-inner'}],
        ['div', {
            'class': 'spinner-dots'
          },
          ['span'],
          ['span'],
          ['span']
        ]
      ],
      [
        'p', {
          'id': LoadingView.ID_MESSAGE
        }
      ]
    ];
  }
};

LoadingView.ID = 'loading';
LoadingView.ID_MESSAGE = 'loading-message';
