'use strict';

class LocationPage extends AbstractPage {
  constructor() {
    super(LocationPage.ID);
    this.registerListeners_();
  }

  render() {
    renderTemplateIfNeeded(this.viewElement,
                           this.Templates.view(geo.list,
                                               geo.countryCode));
  }

  registerHandlers_() {
    super.registerHandlers_();

    EventHandler.register('click', LocationPage.HANDLER_CHANGE,
                          bindOnce(this.onChanged_, this));
  }

  unregisterHandlers_() {
    super.unregisterHandlers_();
    EventHandler.unregister('click', LocationPage.HANDLER_CHANGE,
                            bindOnce(this.onChanged_, this));
  }

  registerListeners_() {
    super.registerListeners_();
    Geo.Events.onChanged.addListener(bindOnce(this.onGeoChanged_, this));
  }

  unregisterListeners_() {
    super.unregisterListeners_();
    Geo.Events.onChanged.removeListener(bindOnce(this.onGeoChanged_, this));
  }

  onChanged_(event, target) {
    if (target && target.dataset.code) {
      LocationPage.Events.onSelected.dispatch(target.dataset.code);
    }
  }

  onGeoChanged_() {
    if (this.isVisible) {
      this.render();
    }
  }
};

LocationPage.Templates = class {
  static view(list, countryCode) {
    return [
      ['h3', i18n.get('locationPageTitle')],
      [
        'div', {
          'class': 'list'
        },
        list.map(function(item) {
          var isSelected = item.country_code === countryCode;
          return [
            'div', {
              'class': [
                'list-item',
                'is-selectable',
                (isSelected ? 'is-selected': '')
              ].join(' ').trim(),
              'data-handler': LocationPage.HANDLER_CHANGE,
              'data-code': !isSelected ? item.country_code : null
            },
            item.country
          ];
        })
      ]
    ];
  }
};

LocationPage.ID = 'location';

LocationPage.HANDLER_CHANGE = 'location-change';

LocationPage.Events = {
  onSelected: new chrome.Event()
};
