'use strict';

class LoginForm extends AbstractForm {
  constructor() {
    super(LoginForm.ID);
    this.title_= i18n.get('signInUpTitleLogin');
    this.cachedData_ = {
      'email': '',
      'password': ''
    };

    this.registerListeners_();
  }

  registerListeners_() {
    super.registerListeners_();

    User.Events.onLoggedIn.addListener(bindOnce(this.onUserLoggedIn_, this));
    User.Events.onCanRegisterChanged.addListener(
        bindOnce(this.onUserCanRegisterChanged_, this));
  }

  unregisterListeners_() {
    super.unregisterListeners_();

    User.Events.onLoggedIn.removeListener(bindOnce(this.onUserLoggedIn_, this));
    User.Events.onCanRegisterChanged.removeListener(
        bindOnce(this.onUserCanRegisterChanged_, this));
  }

  get data() {
    return {
      'email': this.form.email.value,
      'password': this.form.password.value
    };
  }

  onUserCanRegisterChanged_() {
    if (this.isVisible) {
      this.render();
    }
  }

  onUserLoggedIn_() {
    this.cachedData_ = {
      'email': '',
      'password': ''
    }
  }

  onFormSubmited_(event, target) {
    super.onFormSubmited_(event, target);

    this.cachedData_ = this.data;

    if (this.isValid) {
      LoginForm.Events.onSubmited.dispatch(this.data);
    } else {
      if (this.form.email.value.length && !this.form.email.checkValidity()) {
        this.addFormElementError_(this.form.email,
            i18n.get('signInUpErrorEmailInvalid'));
      }
    }
  }

  render() {
    renderTemplateIfNeeded(this.viewElement,
        this.Templates.view({
          'email': user.email ? user.email : this.cachedData_.email,
          'password': this.cachedData_.password,
          'canRegister': user.canRegister
        }));
  }
};

LoginForm.Templates = class extends AbstractForm.Templates {
  static header() {
    return [
      'h1',
      ['strong', i18n.get('signInUpTitle')],
      ['span', i18n.get('signInUpTitleLogin')]
    ];
  }

  static linkGoRegister() {
    return [
      'span', {
        'class': 'link',
        'tabindex': '0',
        'data-handler': AbstractPageView.HANDLER_NAVIGATE_TO_PAGE,
        'data-page': RegisterForm.ID
      },
      i18n.get('signInUpLinkGoToRegister')
    ];
  }

  static buttonSubmit() {
    return ['button', i18n.get('buttonLogin')];
  }

  static summary(canRegister) {
    return [
      'div', {
        'class': 'summary'
      },
      (canRegister ? this.linkGoRegister() : null),
      this.buttonSubmit()
    ];
  }

  static form(data) {
    var form = super.form();

    form.push(this.formElement({
          'type': 'email',
          'name': 'email',
          'value':  data.email
        }, i18n.get('labelEmail')));

    var passwordElement = this.formElement({
          'type': 'password',
          'name': 'password',
          'value':  data.password
        }, i18n.get('labelPassword'));

    passwordElement.push([
      'div', {
        'class': 'push-to-end'
      },
      [
        'a', {
          'class': 'link',
          'target': '_blank',
          'href': Url.USER_RESTORE_PASSWORD
        },
        i18n.get('signInUpLinkGoToPasswordReset')
      ]
    ]);

    form.push(passwordElement);

    form.push(this.summary(data.canRegister));

    return form;
  }

  static view(data) {
    return [
      this.header(),
      this.form(data)
    ];
  }
};

LoginForm.Events = {
  onSubmited: new chrome.Event()
};

LoginForm.ID = 'login';
