'use strict';

class NumberRewardForm extends AbstractForm {
  constructor() {
    super(NumberRewardForm.ID);

    this.cachedData_ = {
      'phone': ''
    };

    this.registerListeners_();
  }

  get rewardItem() {
    return rewards.getItem(Reward.REWARD_NUMBER_INPUT);
  }

  get data() {
    return {
      'phone': this.form.phone.value
    };
  }

  registerHandlers_() {
    super.registerHandlers_();
    EventHandler.register('click', NumberRewardForm.HANDLER_SKIP,
                          bindOnce(this.onSkip_, this));
  }

  unregisterHandlers_() {
    super.unregisterHandlers_();
    EventHandler.unregister('click', NumberRewardForm.HANDLER_SKIP,
                            bindOnce(this.onSkip_, this));
  }

  onFormSubmited_(event, target) {
    super.onFormSubmited_(event, target);

    var data = this.data;
    this.cachedData_ = data;

    if (this.isValid) {
      toolbarState.setLoading(NumberRewardForm.ID, true);
      api.sendSMS(data.phone).then(function() {
        toolbarState.setLoading(NumberRewardForm.ID, false);
        this.rewardItem.isCompleted = true;
      }.bind(this)).catch(function(details) {
        toolbarState.setLoading(NumberRewardForm.ID, false);
        this.addFormError(details);
      }.bind(this));
    }
  }

  onSkip_() {
    this.rewardItem.isCompleted = true;
  }

  render() {
    renderTemplateIfNeeded(this.viewElement,
        this.Templates.view(this.rewardItem, this.cachedData_));
  }
};

NumberRewardForm.Templates = class extends AbstractForm.Templates {
  static form(data) {
    var form = super.form();

    form.push(['h5', i18n.get('numberRewardFormNumberSectionTitle')]);
    form.push(this.formElement({
          'type': 'text',
          'name': 'phone',
          'value': data.phone
        },
        i18n.get('labelPhone'),
        i18n.get('numberRewardFormNumberSectionHint')));

    form.push(this.summary());

    return form;
  }

  static skipButton() {
    return [
      'span', {
        'class': 'link',
        'data-handler': NumberRewardForm.HANDLER_SKIP
      },
      i18n.get('buttonNotNow')
    ];
  }

  static buttonSubmit() {
    return ['button', i18n.get('buttonSendLink')];
  }

  static summary() {
    return [
      'div', {
        'class': 'summary'
      },
      this.skipButton(),
      this.buttonSubmit()
    ];
  }

  static view(reward, data) {
    return [
      [
        'article',
        [
          'h1',
          i18n.get('welcomeNumberRewardTitle', [
            reward.amount + Reward.REWARD_AMOUNT_UNIT
          ])
        ],
        [
          'p',
          i18n.get('rewardNumberInputDescription', [
            reward.amount + Reward.REWARD_AMOUNT_UNIT
          ])
        ]
      ],
      this.form(data)
    ];
  }
};

NumberRewardForm.ID = 'number-reward';
NumberRewardForm.HANDLER_SKIP = 'number-reward-skip';
