'use strict';

class RewardsPage extends AbstractPage {
  constructor() {
    super(RewardsPage.ID);
    this.registerListeners_();
  }

  render() {
    var list = rewards.activeItems;

    list = list.reduce(function(list, item) {
      var group = list[list.length - 1];

      if (!group || group.length === 2) {
        group = [item];
        list.push(group);
      } else {
        group.push(item);
      }
      return list;
    }, []);

    renderTemplateIfNeeded(this.viewElement, this.Templates.view(list));
  }

  registerHandlers_() {
    super.registerHandlers_();

    EventHandler.register('click', RewardsPage.HANDLER_SELECT,
        this.onSelect_);
  }

  unregisterHandlers_() {
    super.unregisterHandlers_();
    EventHandler.unregister('click', RewardsPage.HANDLER_SELECT,
        this.onSelect_);
  }

  registerListeners_() {
    super.registerListeners_();
    Rewards.Events.onChanged.addListener(bindOnce(this.onListChanged_, this));
    Reward.Events.onChanged.addListener(bindOnce(this.onRewardChanged_, this));
  }

  unregisterListeners_() {
    super.unregisterListeners_();
    Rewards.Events.onChanged.removeListener(
        bindOnce(this.onListChanged_, this));
    Reward.Events.onChanged.removeListener(
        bindOnce(this.onRewardChanged_, this));
  }

  onSelect_(event, target) {
    if (target && target.dataset.url) {
      Url.openAutorized(target.dataset.url);
    }
  }

  onListChanged_() {
    if (this.isVisible) {
      this.render();
    }
  }

  onRewardChanged_() {}
};

RewardsPage.Templates = class {
  static view(list) {
    return [
      [
        'div', {
          'class': 'list'
        },
        list.map(function(rewards) {
          return [
            'div', {
              'class': 'list-item'
            },
            rewards.map(function(reward) {
              return [
                'div', {
                  'data-handler': RewardsPage.HANDLER_SELECT,
                  'data-url': reward.url
                },
                [
                  'img', {
                    'src': Url.REWARDS_ICON_SRC.replace('$1', reward.icon)
                  }
                ],
                ['span', reward.description],
                (reward.amount ? [
                      'div',
                      '+' + reward.amount + Reward.REWARD_AMOUNT_UNIT
                    ] : null)
              ];
            })
          ]
        })
      ],
      AbstractDataHomeView.Templates.upgradeButton()
    ];
  }
};

RewardsPage.ID = 'rewards';

RewardsPage.HANDLER_SELECT = 'rewards-select';