'use strict';

class SideMenuPage extends AbstractPage {
  constructor() {
    super(SideMenuPage.ID);

    this.pagesList_ = [
      {
        'label': i18n.get('buttonSupport'),
        'data-handler': SideMenuPage.HANDLER_NAVIGATE_TO_PAGE,
        'data-page': Url.SUPPORT
      },
      {
        'label': i18n.get('buttonAbout'),
        'data-handler': SideMenuPage.HANDLER_NAVIGATE_TO_PAGE,
        'data-page': Url.ABOUT_US
      },
      {
        'label': i18n.get('buttonTermsOfService'),
        'data-handler': SideMenuPage.HANDLER_NAVIGATE_TO_PAGE,
        'data-page': Url.TERMS_OF_SERVICE
      },
      {
        'label': i18n.get('buttonPrivacyPolicy'),
        'data-handler': SideMenuPage.HANDLER_NAVIGATE_TO_PAGE,
        'data-page': Url.PRIVACY_POLICY
      }
    ];

    this.registerListeners_();
  }

  onNavigateToPage_(event, target) {
    if (target && target.dataset.page) {
      Url.open(target.dataset.page);
    }
  }

  onLogout_() {
    this.hide();
    user.logout();
  }

  onTurnOnOff_() {
    proxy.isEnabled = !proxy.isEnabled;
  }

  registerHandlers_() {
    super.registerHandlers_();

    EventHandler.register('click', SideMenuPage.HANDLER_NAVIGATE_TO_PAGE,
                          bindOnce(this.onNavigateToPage_, this));
    EventHandler.register('click', SideMenuPage.HANDLER_LOGOUT,
                          bindOnce(this.onLogout_, this));
    EventHandler.register('click', SideMenuPage.HANDLER_TURN_ON_OFF,
                          bindOnce(this.onTurnOnOff_, this));
  }

  unregisterHandlers_() {
    super.unregisterHandlers_();
    EventHandler.unregister('click', SideMenuPage.HANDLER_NAVIGATE_TO_PAGE,
                            bindOnce(this.onNavigateToPage_, this));
    EventHandler.unregister('click', SideMenuPage.HANDLER_LOGOUT,
                            bindOnce(this.onLogout_, this));
    EventHandler.unregister('click', SideMenuPage.HANDLER_TURN_ON_OFF,
                            bindOnce(this.onTurnOnOff_, this));
  }

  registerListeners_() {
    super.registerListeners_();
    var onChanged_ = bindOnce(this.onChanged_, this);
    Proxy.Events.onEnabledChanged.addListener(onChanged_);
    User.Events.onChanged.addListener(onChanged_);
  }

  unregisterListeners_() {
    super.unregisterListeners_();
    var onChanged_ = bindOnce(this.onChanged_, this);
    Proxy.Events.onEnabledChanged.removeListener(onChanged_);
    User.Events.onChanged.addListener(onChanged_);
  }

  onChanged_() {
    if (this.isVisible) {
      this.render();
    }
  }

  render() {
    renderTemplateIfNeeded(
        this.viewElement,
        this.Templates.view(this.pagesList_, proxy.isEnabled, user.isLoggedIn));
  }
};

SideMenuPage.Templates = class {
  static listItem(data) {
    data.class = [
      'list-item',
      data.class ? data.class : 'is-link'
    ].join(' ').trim();
    var label = data.label;

    return [
      'div',
      data,
      label
    ];
  }

  static view(list, isProxyOn, isUserLoggedIn) {
    return [
      [
        'header',
        [
          'div', {
            'class': ['turn-on-off-link', isProxyOn ? 'is-on' : ''].join(' ')
                .trim(),
            'data-handler': SideMenuPage.HANDLER_TURN_ON_OFF
          },
          i18n.get(isProxyOn ? 'buttonTurnOff': 'buttonTurnOn')
        ]
      ],
      [
        'div', {
          'class': 'list'
        },
        list.map(this.listItem)
      ],
      [
        'footer',
        isUserLoggedIn ? [
          'div', {
            'class': 'logout-link',
            'data-handler': SideMenuPage.HANDLER_LOGOUT
          },
          i18n.get('buttonLogout')
        ] : null,
        [
          'div', {
            'class': 'made-in'
          },
          i18n.getTemplate('sideMenuMadeIn', {
                '$1': [
                  'img', {
                    'src': '/images/love.svg'
                  }
                ]
              })
        ]
      ]
    ];
  }
};

SideMenuPage.ID = 'side-menu';
SideMenuPage.HANDLER_NAVIGATE_TO_PAGE = 'side-menu-navigate-to-page';
SideMenuPage.HANDLER_LOGOUT = 'side-menu-logout';
SideMenuPage.HANDLER_TURN_ON_OFF = 'side-menu-turn-on-off';