'use strict';

class SignView extends AbstractPageView {
  constructor() {
    super(SignView.ID);

    this.title_ = i18n.get('signInUpTitle');

    this.registerListeners_();
    this.onUserChanged_();
  }

  get defaultPageID() {
    return !user.canRegister || Boolean(user.email) ? LoginForm.ID :
        RegisterForm.ID;
  }

  render() {
    this.renderSubpage(this.Templates.page());
  }

  registerHandlers_() {
    super.registerHandlers_();
    LoginForm.Events.onSubmited.addListener(bindOnce(this.onFormLogin_, this));
    RegisterForm.Events.onSubmited.addListener(
        bindOnce(this.onFormRegister_, this));
  }

  unregisterHandlers_() {
    super.unregisterHandlers_();
    LoginForm.Events.onSubmited.removeListener(
        bindOnce(this.onFormLogin_, this));
    RegisterForm.Events.onSubmited.removeListener(
        bindOnce(this.onFormRegister_, this));
  }

  registerListeners_() {
    super.registerListeners_();

    User.Events.onCanRegisterChanged.addListener(
        bindOnce(this.onUserCanRegisterChanged_, this));
    User.Events.onChanged.addListener(bindOnce(this.onUserChanged_, this));
  }

  unregisterListeners_() {
    super.unregisterListeners_();

    User.Events.onCanRegisterChanged.removeListener(
        bindOnce(this.onUserCanRegisterChanged_, this));
    User.Events.onChanged.removeListener(bindOnce(this.onUserChanged_, this));
  }

  onFormLogin_(data) {
    user.login(data.email, data.password).catch(this.addFormError_.bind(this));
  }

  onFormRegister_(data) {
    user.register(data.email, data.password).then(doNothing)
        .catch(this.addFormError_.bind(this));
  }

  onUserCanRegisterChanged_() {
    if (this.isVisible) {
      if (this.currentPageID === RegisterForm.ID && !user.canRegister) {
        this.currentPageID = LoginForm.ID;
      } else {
        this.render();
      }
    }
  }

  addFormError_(details) {
    if (this.currentPageID && details) {
      this.currentPage.addFormError(details);
    }
  }

  onUserChanged_() {
    this.canShow = !user.isLoggedIn;
  }
};

SignView.Templates = class extends AbstractPageView.Templates {};

SignView.ID = 'sign';
