'use strict';

class WelcomeRewardsView extends AbstractPageView {
  constructor() {
    super(WelcomeRewardsView.ID);
    this.title_ = i18n.get('welcomeTitle');

    this.registerListeners_();
    this.onRewardCompleteChanged_();
  }

  get defaultPageID() {
    return this.welcomeReward.isCompleted ? NumberRewardForm.ID :
        WelcomeRewardPage.ID;
  }

  get welcomeReward() {
    return rewards.getItem(Reward.REWARD_WELCOME);
  }

  get numberInputReward() {
    return rewards.getItem(Reward.REWARD_NUMBER_INPUT);
  }

  render() {
    if (this.currentPageID === WelcomeRewardPage.ID) {
      this.renderSubpage(this.Templates.welcomeRewardView());
    } else {
      this.renderSubpage(this.Templates.numberInputRewardView());
    }
  }

  registerHandlers_() {
    super.registerHandlers_();
    EventHandler.register('click',
                          WelcomeRewardsView.HANDLER_WELCOME_REWARD_SKIP,
                          bindOnce(this.onWelcomeRewardSkip_, this));
  }

  unregisterHandlers_() {
    super.unregisterHandlers_();
    EventHandler.unregister('click',
                            WelcomeRewardsView.HANDLER_WELCOME_REWARD_SKIP,
                            bindOnce(this.onWelcomeRewardSkip_, this));
  }

  registerListeners_() {
    super.registerListeners_();
    Reward.Events.onChanged.addListener(
        bindOnce(this.onRewardCompleteChanged_, this));
  }

  unregisterListeners_() {
    super.unregisterListeners_();
    Reward.Events.onChanged.removeListener(
        bindOnce(this.onRewardCompleteChanged_, this));
  }

  onWelcomeRewardSkip_() {
    this.welcomeReward.isCompleted = true;
  }

  onRewardCompleteChanged_() {
    this.currentPageID = this.defaultPageID;
    this.canShow = !rewards.areWelcomeRewardsCompleted;
  }
};

WelcomeRewardsView.Templates = class extends AbstractPageView.Templates {
  static header() {
    return WelcomeRewardPage.Templates.header();
  }

  static numberInputRewardView() {
    return super.page();
  }

  static welcomeRewardView() {
    return [
      super.page(),
      [
        'div', {
          'class': 'pagination'
        },
        [
          'div', {
            'class': 'pagination-next',
            'data-handler': WelcomeRewardsView.HANDLER_WELCOME_REWARD_SKIP
          },
          ['span', i18n.get('buttonNext')]
        ]
      ]
    ];
  }
};

WelcomeRewardsView.ID = 'welcome-rewards';
WelcomeRewardsView.HANDLER_WELCOME_REWARD_SKIP = 'welcome-reward-skip';
