'use strict';

class LandingSignView extends SignView {
  constructor() {
    super();
    this.title_ = i18n.get('landingPageSignTitle');
  }

  registerHandlers_() {
    super.registerHandlers_();
    EventHandler.register('click', LandingSignView.HANDLER_EXPAND_ONBOARDING,
                          bindOnce(this.onExpand_, this));
  }

  unregisterHandlers_() {
    super.unregisterHandlers_();
    EventHandler.register('click', LandingSignView.HANDLER_EXPAND_ONBOARDING,
                          bindOnce(this.onExpand_, this));
  }

  onExpand_(event, target) {
    if (!target.classList.contains(LandingSignView.CLASS_EXPANDED)) {
      target.classList.add(LandingSignView.CLASS_EXPANDED);
      app.isOnboardingCompleted = true;
    }
  }

  render() {
    this.renderSubpage(this.Templates.page());

    renderTemplateIfNeeded(this.headerElement,
        this.currentPage.Templates.header());

    renderTemplateIfNeeded(this.footerElement,
        this.currentPageID === LandingRegisterForm.ID ?
            this.Templates.footer() : []);
  }
};

LandingSignView.Templates = class extends SignView.Templates {
  static footer() {
    return [
      [
        'section', {
          'id': 'register-onboarding',
          'data-handler': LandingSignView.HANDLER_EXPAND_ONBOARDING
        },
        ['h3', i18n.get('landingPageSignRegisterOnBoardingTitle')],
        [
          'div',
          [
            'article',
            ['img', {'src': '/images/lock.svg'}],
            ['h4', i18n.get('onboardingSecureNetworkTitle')],
            ['p', i18n.get('onboardingSecureNetworkText')]
          ],
          [
            'article',
            ['img', {'src': '/images/safe.svg'}],
            ['h4', i18n.get('onboardingProtectPrivacyTitle')],
            ['p', i18n.get('onboardingProtectPrivacyText')]
          ],
          [
            'article',
            ['img', {'src': '/images/key.svg'}],
            ['h4', i18n.get('onboardingProtectIdentityTitle')],
            ['p', i18n.get('onboardingProtectIdentityText')]
          ],
          [
            'article',
            ['img', {'src': '/images/globe.svg'}],
            ['h4', i18n.get('onboardingUnblockNetworkTitle')],
            ['p', i18n.get('onboardingUnblockNetworkText')]
          ]
        ]
      ],
      Landing.Templates.moreForNewUser()
    ]
  }
};

LandingSignView.ID = SignView.ID;
LandingSignView.CLASS_EXPANDED = 'expanded';
LandingSignView.HANDLER_EXPAND_ONBOARDING = 'landing-sign-expand-onboarding';
