'use strict';

class Landing extends AbstractSite {
  constructor() {
    super(Landing.ID);

    this.addPage(new LocationPage());
    this.addPage(new LandingLoginForm());
    this.addPage(new LandingRegisterForm());
    this.addPage(new LandingWelcomeRewardPage());
    this.addPage(new LandingNumberRewardForm());

    this.addView(new ErrorView());
    this.addView(new LoadingView());
    this.addView(new LandingSignView());
    this.addView(new LandingWelcomeRewardsView());

    this.initialize();
  }

  get stepsElement() {
    return this.siteElement.querySelector(idToSelector(Landing.ID_STEPS));
  }

  get isNewUserView() {
    return (this.currentView &&
      (this.currentView.id === LandingWelcomeRewardsView.ID ||
      (this.currentView.id === LandingSignView.ID &&
      this.currentView.currentPageID === LandingRegisterForm.ID)));
  }

  get step() {
    var selected = this.stepsElement.querySelector(classesToSelector(
        [Landing.CLASS_STEP, Landing.CLASS_STEP_SELECTED], true));

    if (selected && !isNaN(selected.dataset.step)) {
      return parseInt(selected.dataset.step);
    }
  }

  set step(value) {
    if (this.step === value) {
      return;
    }

    var selected = this.stepsElement.querySelector(classesToSelector(
        [Landing.CLASS_STEP, Landing.CLASS_STEP_SELECTED], true));

    if (selected) {
      selected.classList.remove(Landing.CLASS_STEP_SELECTED);
    }

    var steps = site.stepsElement.querySelectorAll(classesToSelector(
        Landing.CLASS_STEP));

    value = value.toString();
    for_each(steps, function(item) {
      if (item.dataset.step === value) {
        item.classList.add(Landing.CLASS_STEP_SELECTED);
      }
    });
  }

  render() {
    this.siteElement_ = document.body.appendTemplate(this.Templates.view(
        this.id));
  }

  onCanShowChanged_() {
    super.onCanShowChanged_();

    if (user.isLoggedIn && rewards.areWelcomeRewardsCompleted) {
      this.close();
    }
  }

  onPageViewChanged_() {
    super.onPageViewChanged_();
    if (!user.isLoggedIn) {
      this.step = Landing.STEP_SING_IN_UP;
    } else if (!rewards.areWelcomeRewardsCompleted) {
      this.step = Landing.STEP_WELCOME;
    }
    this.stepsElement.hidden = !this.isNewUserView;
  }

  close() {
    if (this.currentView) {
      this.currentView.isVisible = false;
    }
    Url.close(window.location.href);
  }
};

Landing.Templates = class extends AbstractSite.Templates {
  static moreForNewUser() {
    return [
      'section', {
        'id': 'new-user-footer'
      },
      [
        'h3',
        ['span', i18n.get('landingPageFooterTitle')],
        [
          'a', {
            'class': 'link',
            'href': Url.BASE
          },
          i18n.get('landingPageFooterLink')
        ]
      ]
    ]
  }

  static header() {
    return [
      'header',
      ['div', {'class': 'icon-logo'}],
      ['div', {'id': AbstractSite.ID_HEADER}],
      [
        'div', {
          'id': Landing.ID_STEPS,
          'hidden': ''
        },
        [
          'div', {
            'class': [
              Landing.CLASS_STEP,
              Landing.CLASS_STEP_SELECTED
            ].join(' '),
            'data-step': Landing.STEP_SING_IN_UP.toString()
          },
          i18n.get('landingPageStep', [Landing.STEP_SING_IN_UP])
        ],
        [
          'div', {
            'class': Landing.CLASS_STEP,
            'data-step': Landing.STEP_WELCOME.toString()
          },
          i18n.get('landingPageStep', [Landing.STEP_WELCOME])
        ]
      ]
    ]
  }

  static view(id) {
    return [
      'div', {
        'id': id + '-wrap'
      },
      super.view(id)
    ];
  }
};

Landing.ID = 'landing';
Landing.ID_STEPS = 'landing-steps';
Landing.CLASS_STEP = 'step';
Landing.CLASS_STEP_SELECTED = 'selected';
Landing.STEP_SING_IN_UP = 1;
Landing.STEP_WELCOME = 2;

document.addEventListener('DOMContentLoaded', function() {
  chrome.runtime.getBackgroundPage(function(backgroundPage) {
    window.Url = backgroundPage.Url;
    window.api = backgroundPage.api;
    window.App = backgroundPage.App;
    window.app = backgroundPage.app;
    window.analytics = backgroundPage.analytics;
    window.User = backgroundPage.User;
    window.user = backgroundPage.user;
    window.Device = backgroundPage.Device;
    window.device = backgroundPage.device;
    window.Reward = backgroundPage.Reward;
    window.Rewards = backgroundPage.Rewards;
    window.rewards = backgroundPage.rewards;
    window.networkTasks = backgroundPage.networkTasks;
    window.Geo = backgroundPage.Geo;
    window.geo = backgroundPage.geo;
    window.Errors = backgroundPage.Errors;
    window.errors = backgroundPage.errors;
    window.OffRoad = backgroundPage.OffRoad;
    window.offroad = backgroundPage.offroad;
    window.sessionCookie = backgroundPage.sessionCookie;
    window.Proxy = backgroundPage.Proxy;
    window.proxy = backgroundPage.proxy;

    window.ToolbarState = backgroundPage.ToolbarState;
    window.toolbarState = backgroundPage.toolbarState;

    window.site = new Landing();
  });
});