'use strict';

class OnBoardingView extends AbstractView {
  constructor() {
    super(OnBoardingView.ID);
    settings.register(OnBoardingView.SETTING_STEP, 0);

    this.registerListeners_();
    this.onOnboardingSettingChanged_();
  }

  get step() {
    return settings.get(OnBoardingView.SETTING_STEP);
  }

  set step(value) {
    if (value === 2) {
      app.isOnboardingCompleted = true;
      return;
    }

    settings.set(OnBoardingView.SETTING_STEP, value);
    if (this.isVisible) {
      this.render();
    }
  }

  prev() {
    this.step = Math.max(0, this.step - 1);
  }

  next() {
    this.step = Math.min(2, this.step + 1);
  }

  render() {
    renderTemplateIfNeeded(this.viewElement, this.Templates.step(this.step));
  }

  registerHandlers_() {
    super.registerHandlers_();
    EventHandler.register('click', OnBoardingView.HANDLER_PREV,
                          bindOnce(this.prev, this));
    EventHandler.register('click', OnBoardingView.HANDLER_NEXT,
                          bindOnce(this.next, this));
  }

  unregisterHandlers_() {
    super.unregisterHandlers_();
    EventHandler.unregister('click', OnBoardingView.HANDLER_PREV,
                          bindOnce(this.prev, this));
    EventHandler.unregister('click', OnBoardingView.HANDLER_NEXT,
                          bindOnce(this.next, this));
  }

  registerListeners_() {
    super.registerListeners_();

    App.Events.onOnboardingCompleteChanged.addListener(
        bindOnce(this.onOnboardingSettingChanged_, this));
  }

  unregisterListeners_() {
    super.unregisterListeners_();

    App.Events.onOnboardingCompleteChanged.removeListener(
        bindOnce(this.onOnboardingSettingChanged_, this));
  }

  onOnboardingSettingChanged_() {
    this.canShow = !app.isOnboardingCompleted;
  }
};

OnBoardingView.Templates = class {
  static step(step) {
    var view = [
      'div',
      {'class': 'step'},
      ['h3', i18n.get('onboardingTitle')]
    ];

    if (step === 0) {
      view.push([
        'article',
        ['img', {'src': '/images/lock.svg'}],
        ['h4', i18n.get('onboardingSecureNetworkTitle')],
        ['p', i18n.get('onboardingSecureNetworkText')]
      ]);

      view.push([
        'article',
        ['img', {'src': '/images/safe.svg'}],
        ['h4', i18n.get('onboardingProtectPrivacyTitle')],
        ['p', i18n.get('onboardingProtectPrivacyText')]
      ]);
    }

    if (step === 1) {
      view.push([
        'article',
        ['img', {'src': '/images/key.svg'}],
        ['h4', i18n.get('onboardingProtectIdentityTitle')],
        ['p', i18n.get('onboardingProtectIdentityText')]
      ]);

      view.push([
        'article',
        ['img', {'src': '/images/globe.svg'}],
        ['h4', i18n.get('onboardingUnblockNetworkTitle')],
        ['p', i18n.get('onboardingUnblockNetworkText')]
      ]);
    }

    view.push(this.pagination(step));
    return view;
  }

  static pagination(step) {
    var prevButton = [
      'div', {
        'class': 'pagination-prev',
        'data-handler': OnBoardingView.HANDLER_PREV
      },
      ['span', i18n.get('buttonPrev')]
    ];

    var nextButton = [
      'div', {
        'class': 'pagination-next',
        'data-handler': OnBoardingView.HANDLER_NEXT
      },
      ['span', i18n.get('buttonNext')]
    ];

    return [
      'div', {
        'class': 'pagination'
      },
      step !== 0 ? prevButton : null,
      nextButton
    ];
  }
};

OnBoardingView.ID = 'onboarding';
OnBoardingView.HANDLER_NEXT = 'onboarding-next';
OnBoardingView.HANDLER_PREV = 'onboarding-prev';
OnBoardingView.SETTING_STEP = 'onboarding.step';
