'use strict';

class UnlimitedHomeView extends AbstractHomeView {
  constructor() {
    super(UnlimitedHomeView.ID);

    this.homePages_.push(ExpiredPlanPage.ID);
    this.homePages_.push(UnlimitedPlanPage.ID);

    this.registerListeners_();
    this.onCanShowChanged_();
  }

  get defaultHomePageID() {
    if (proxy.isEnabled) {
      if (dataPlan.isExpired) {
        return ExpiredPlanPage.ID;
      } else {
        return UnlimitedPlanPage.ID;
      }
    } else {
      return super.defaultHomePageID;
    }
  }

  get isShowable() {
    return dataPlan.isUnlimited;
  }

  get templateData_() {
    var data = super.templateData_;
    data.isExpired = dataPlan.isExpired;

    return data;
  }

  registerHandlers_() {
    super.registerHandlers_();
    EventHandler.register('click', UnlimitedHomeView.HANDLER_UPGRADE,
                          bindOnce(this.onUpgrade_, this));
    EventHandler.register('click', UnlimitedHomeView.HANDLER_DOWNGRADE,
                          bindOnce(this.onDowngrade_, this));
  }

  unregisterHandlers_() {
    super.unregisterHandlers_();
    EventHandler.unregister('click', UnlimitedHomeView.HANDLER_UPGRADE,
                            bindOnce(this.onUpgrade_, this));
    EventHandler.unregister('click', UnlimitedHomeView.HANDLER_DOWNGRADE,
                            bindOnce(this.onDowngrade_, this));
  }

  onUpgrade_() {
    dataPlan.purchase();
  }

  onDowngrade_() {
    dataPlan.downgrade();
  }
};

UnlimitedHomeView.Templates = class extends AbstractHomeView.Templates {
  static switchToFreeButton() {
    return [
      'button', {
        'class': 'background',
        'data-handler': UnlimitedHomeView.HANDLER_DOWNGRADE
      },
      i18n.get('buttonSwitchToFree')
    ];
  }

  static renewButton() {
    return [
      'button', {
        'class': 'highlighted',
        'data-handler': UnlimitedHomeView.HANDLER_UPGRADE
      },
      i18n.get('buttonRenew')
    ];
  }

  static view(data) {
    var template = super.view(data);

    if (data.isExpired && data.isConnected) {
      template.push(this.switchToFreeButton());
      template.push(this.renewButton());
    }

    return template;
  }
};

UnlimitedHomeView.ID = 'home-unlimited';

UnlimitedHomeView.HANDLER_DOWNGRADE = 'home-unlimited-downgrade';
UnlimitedHomeView.HANDLER_UPGRADE = 'home-unlimited-upgrade';